@@ -247,7 +247,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       content_type nonexistent/invalid
     }
     assert_raise(Fluent::ConfigError) {
-      instance = driver(config).instance
+      driver(config)
     }
   end
 
@@ -256,7 +256,16 @@ class ElasticsearchOutput < Test::Unit::TestCase
       max_retry_putting_template -3
     }
     assert_raise(Fluent::ConfigError) {
-      instance = driver(config).instance
+      driver(config)
+    }
+  end
+
+  test 'invalid specification of times of retrying get es version' do
+    config = %{
+      max_retry_get_es_version -3
+    }
+    assert_raise(Fluent::ConfigError) {
+      driver(config)
     }
   end
 
@@ -274,7 +283,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       @log_level warn
       scheme https
     }
-    instance = driver(config, 6).instance
+    driver(config, 6)
     logs = driver.logs
     assert_logs_include(logs, /Detected ES 6.x or above and enabled insecure security/, 1)
   end
@@ -285,7 +294,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       @log_level warn
       scheme https
     }
-    instance = driver(config, 7).instance
+    driver(config, 7)
     logs = driver.logs
     assert_logs_include(logs, /Detected ES 6.x or above and enabled insecure security/, 0)
   end
@@ -296,7 +305,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       validate_client_version true
     }
     assert_nothing_raised do
-      driver(config, 6, "\"6.1.0\"").instance
+      driver(config, 6, "\"6.1.0\"")
     end
   end
 
@@ -306,7 +315,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       validate_client_version true
     }
     assert_raise_message(/Detected ES 7 but you use ES client 5.0/) do
-      driver(config, 7, "\"5.0.5\"").instance
+      driver(config, 7, "\"5.0.5\"")
     end
   end
 
@@ -392,7 +401,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
         create_driver(config)
       end
 
-      assert_equal(connection_resets, 4)
+      assert_equal(4, connection_resets)
     end
   end
 
@@ -706,7 +715,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver(config)
     end
 
-    assert_equal(connection_resets, 4)
+    assert_equal(4, connection_resets)
   end
 
   def test_template_retry_install_does_not_fail
@@ -735,7 +744,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
 
     driver(config)
 
-    assert_equal(connection_resets, 4)
+    assert_equal(4, connection_resets)
   end
 
   def test_templates_create
@@ -1000,6 +1009,21 @@ class ElasticsearchOutput < Test::Unit::TestCase
     assert_equal nil, host1[:path]
   end
 
+  def test_host_and_port_are_ignored_if_specify_hosts
+    config = %{
+      host  logs.google.com
+      port  9200
+      hosts host1:50,host2:100
+    }
+    instance = driver(config).instance
+
+    params = instance.get_connection_options[:hosts]
+    hosts = params.map { |p| p[:host] }
+    ports = params.map { |p| p[:port] }
+    assert(hosts.none? { |h| h == 'logs.google.com' })
+    assert(ports.none? { |p| p == 9200 })
+  end
+
   def test_content_type_header
     stub_request(:head, "http://localhost:9200/").
       to_return(:status => 200, :body => "", :headers => {})
@@ -1193,7 +1217,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     time = Time.parse Date.today.iso8601
     stub_elastic
     driver.run(default_tag: 'test') do
-      driver.feed(time.to_i, sample_record.merge('@target_index' => 'local-override'))
+      driver.feed(time.to_i, sample_record.merge('@target_index' => 'LOCAL-OVERRIDE'))
     end
     # Allthough @target_index has upper-case characters,
     # it should be set as lower-case when sent to elasticsearch.
@@ -1712,7 +1736,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(time, sample_record)
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts.iso8601(9))
+    assert_equal(ts.iso8601(9), index_cmds[1]['@timestamp'])
   end
 
   def test_adds_timestamp_when_include_timestamp
@@ -1724,7 +1748,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(time, sample_record)
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts.iso8601(9))
+    assert_equal(ts.iso8601(9), index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_timestamp_when_included_in_record
@@ -1735,7 +1759,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record.merge!('@timestamp' => ts))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_timestamp_when_included_in_record_without_logstash
@@ -1746,7 +1770,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record.merge!('@timestamp' => ts))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_time_key
@@ -1758,7 +1782,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record.merge!('vtm' => ts))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_time_key_with_float_record
@@ -1772,7 +1796,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record.merge!('vtm' => float_time))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], time.to_datetime.iso8601(3))
+    assert_equal(time.to_datetime.iso8601(3), index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_time_key_with_format
@@ -1785,7 +1809,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record.merge!('vtm' => ts))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], DateTime.parse(ts).iso8601(9))
+    assert_equal(DateTime.parse(ts).iso8601(9), index_cmds[1]['@timestamp'])
     assert_equal("logstash-2001.02.03", index_cmds[0]['index']['_index'])
   end
 
@@ -1796,13 +1820,13 @@ class ElasticsearchOutput < Test::Unit::TestCase
     stub_elastic
     ts = "2001-02-03 13:14:01.673+02:00"
     time = Time.parse(ts)
-    current_zone_offset = Time.now.to_datetime.offset
+    current_zone_offset = Time.new(2001, 02, 03).to_datetime.offset
     float_time = time.to_f
     driver.run(default_tag: 'test') do
       driver.feed(sample_record.merge!('vtm' => float_time))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], DateTime.parse(ts).new_offset(current_zone_offset).iso8601(9))
+    assert_equal(DateTime.parse(ts).new_offset(current_zone_offset).iso8601(9), index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_time_key_with_format_without_logstash
@@ -1816,7 +1840,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record.merge!('vtm' => ts))
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], DateTime.parse(ts).iso8601(9))
+    assert_equal(DateTime.parse(ts).iso8601(9), index_cmds[1]['@timestamp'])
     assert_equal("test", index_cmds[0]['index']['_index'])
   end
 
@@ -1842,7 +1866,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     end
     assert_equal("logstash-2001.02.03", index_cmds[0]['index']['_index'])
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
   def test_uses_custom_time_key_format_without_logstash
@@ -1856,7 +1880,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     end
     assert_equal("test", index_cmds[0]['index']['_index'])
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
   data(:default => nil,
@@ -1879,7 +1903,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
 
     assert_equal(index, index_cmds[0]['index']['_index'])
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
 
@@ -1893,7 +1917,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     end
     assert_equal("logstash-2001.11.29", index_cmds[0]['index']['_index'])
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], ts)
+    assert_equal(ts, index_cmds[1]['@timestamp'])
   end
 
   def test_uses_nanosecond_precision_by_default
@@ -1904,7 +1928,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(time, sample_record)
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], Time.at(time).iso8601(9))
+    assert_equal(Time.at(time).iso8601(9), index_cmds[1]['@timestamp'])
   end
 
   def test_uses_subsecond_precision_when_configured
@@ -1916,7 +1940,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(time, sample_record)
     end
     assert(index_cmds[1].has_key? '@timestamp')
-    assert_equal(index_cmds[1]['@timestamp'], Time.at(time).iso8601(3))
+    assert_equal(Time.at(time).iso8601(3), index_cmds[1]['@timestamp'])
   end
 
   def test_doesnt_add_tag_key_by_default
@@ -1934,7 +1958,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record)
     end
     assert(index_cmds[1].has_key?('tag'))
-    assert_equal(index_cmds[1]['tag'], 'mytag')
+    assert_equal('mytag', index_cmds[1]['tag'])
   end
 
   def test_adds_id_key_when_configured
@@ -1943,7 +1967,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     driver.run(default_tag: 'test') do
       driver.feed(sample_record)
     end
-    assert_equal(index_cmds[0]['index']['_id'], '42')
+    assert_equal('42', index_cmds[0]['index']['_id'])
   end
 
   class NestedIdKeyTest < self
@@ -1953,7 +1977,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_id'], '42')
+      assert_equal('42', index_cmds[0]['index']['_id'])
     end
 
     def test_adds_nested_id_key_with_dollar_dot
@@ -1962,7 +1986,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_id'], '42')
+      assert_equal('42', index_cmds[0]['index']['_id'])
     end
 
     def test_adds_nested_id_key_with_bracket
@@ -1971,7 +1995,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_id'], '42')
+      assert_equal('42', index_cmds[0]['index']['_id'])
     end
   end
 
@@ -1998,7 +2022,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     driver.run(default_tag: 'test') do
       driver.feed(sample_record)
     end
-    assert_equal(index_cmds[0]['index']['_parent'], 'parent')
+    assert_equal('parent', index_cmds[0]['index']['_parent'])
   end
 
   class NestedParentKeyTest < self
@@ -2008,7 +2032,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_parent'], 'parent')
+      assert_equal('parent', index_cmds[0]['index']['_parent'])
     end
 
     def test_adds_nested_parent_key_with_dollar_dot
@@ -2017,7 +2041,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_parent'], 'parent')
+      assert_equal('parent', index_cmds[0]['index']['_parent'])
     end
 
     def test_adds_nested_parent_key_with_bracket
@@ -2026,7 +2050,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_parent'], 'parent')
+      assert_equal('parent', index_cmds[0]['index']['_parent'])
     end
   end
 
@@ -2054,7 +2078,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_routing'], 'routing')
+      assert_equal('routing', index_cmds[0]['index']['_routing'])
     end
 
     def test_es7
@@ -2063,7 +2087,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(sample_record)
       end
-      assert_equal(index_cmds[0]['index']['routing'], 'routing')
+      assert_equal('routing', index_cmds[0]['index']['routing'])
     end
   end
 
@@ -2074,7 +2098,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_routing'], 'routing')
+      assert_equal('routing', index_cmds[0]['index']['_routing'])
     end
 
     def test_adds_nested_routing_key_with_dollar_dot
@@ -2083,7 +2107,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_routing'], 'routing')
+      assert_equal('routing', index_cmds[0]['index']['_routing'])
     end
 
     def test_adds_nested_routing_key_with_bracket
@@ -2092,7 +2116,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.run(default_tag: 'test') do
         driver.feed(nested_sample_record)
       end
-      assert_equal(index_cmds[0]['index']['_routing'], 'routing')
+      assert_equal('routing', index_cmds[0]['index']['_routing'])
     end
   end
 
@@ -2175,7 +2199,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
         driver.feed(sample_record)
       end
     }
-    assert_equal(connection_resets, 1)
+    assert_equal(1, connection_resets)
   end
 
   def test_reconnect_on_error_enabled
@@ -2201,7 +2225,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
     }
     # FIXME: Consider keywords arguments in #run and how to test this later.
     # Because v0.14 test driver does not have 1 to 1 correspondence between #run and #flush in tests.
-    assert_equal(connection_resets, 1)
+    assert_equal(1, connection_resets)
   end
 
   def test_reconnect_on_error_disabled
@@ -2225,7 +2249,7 @@ class ElasticsearchOutput < Test::Unit::TestCase
         driver.feed(sample_record)
       end
     }
-    assert_equal(connection_resets, 1)
+    assert_equal(1, connection_resets)
   end
 
   def test_bulk_error_retags_when_configured
@@ -2609,8 +2633,8 @@ class ElasticsearchOutput < Test::Unit::TestCase
       driver.feed(sample_record)
     end
 
-    assert_equal(index_cmds.length, 2)
-    assert_equal(index_cmds.first['index']['_index'], nil)
+    assert_equal(2, index_cmds.length)
+    assert_equal(nil, index_cmds.first['index']['_index'])
   end
 
   def test_use_simple_sniffer
